﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Runtime.InteropServices;

namespace CSharpSample
{

    public partial class Form1 : Form
    {
        void InitializeShetabPacker()
        {
            try
            {
                //create packer
                //zip contents will not extract to temp folder, temp folder needed for internal use
                Pakcer = Shetab.Packer.PackerHelper.Create();
                Pakcer.CreateBoxManager(Path.Combine(Path.GetTempPath(), "PackerSample"), out BoxManager);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                throw ex;
            }
        }

        public Form1()
        {
            InitializeComponent();
            InitializeShetabPacker();
        }

        public String ExeFile
        {
            get { return Application.ExecutablePath; }
        }
        public String ExeFolder
        {
            get { return Path.GetDirectoryName(ExeFile); }
        }
        public String ZipFile
        {
            get
            {
                return Path.Combine(ExeFolder, "test.zip");
            }
        }

        Shetab.Packer.IPacker Pakcer;
        Shetab.Packer.IBoxManager BoxManager;
        private void Form1_Load(object sender, EventArgs e)
        {
            // Set up the ToolTip text for the Button and Checkbox.
            toolTip1.SetToolTip(this.ExtractFileButton, "Extract \"sample.htm\" file from test.zip file and save it out of test.zip.\nMake sure \"simple.htm\" file exists into test.zip");
            toolTip1.SetToolTip(this.ExtractFolderButton, "Extract \"sample\" folder from test.zip file and save it out of test.zip.\nMake sure \"simple\" folder exists into test.zip");
            webBrowser1.Navigate(Path.Combine(ExeFolder, @"ReadMe.htm"));
        }

        /// <summary>
        /// Remove mapping of ZipFile in process environment, you can not access file and folder inside of zip file via standard file IO operation
        /// You can write, read the files via ZipStream interface
        /// </summary>
        private void Unmount()
        {
            //Stop Mount if started
            bool isStarted;
            BoxManager.IsMounted(out isStarted);
            if (!isStarted)
                return;

            BoxManager.Unmount();

            //remove all boxes so let them to open as write access again
            BoxManager.RemoveAllBoxes();
        }

        /// <summary>
        /// Map ZipFile to process environment, the process will see the files inside of zip files as normal file and folder
        /// </summary>
        private void Mount()
        {
            //return if already started
            bool isStarted;
            BoxManager.IsMounted(out isStarted);
            if (isStarted)
                return; //already started


            //create zip storage from existing file
            Shetab.Packer.IZipStorage zipStorage;
            Pakcer.OpenZipStorage(ZipFile, "", true, out zipStorage);

            //create BoxManager and add zipStorage to it
            BoxManager.AddBox(zipStorage, ExeFolder);

            //start monitor
            BoxManager.Mount();

            //disable buttons
            AddFileButton.Enabled =
            AddFolderButton.Enabled = false;
            label2.Visible = true;
        }

        private void SelectFileToView_Click(object sender, EventArgs e)
        {
            Mount();

            //OpenDialog
            openFileDialog1.InitialDirectory = ExeFolder;
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
                webBrowser1.Navigate(openFileDialog1.FileName);
        }

        private void AddFileButton_Click(object sender, EventArgs e)
        {
            Shetab.Packer.IZipStorage zipStorage = null; ;

            try
            {
                //Open zip storage
                Pakcer.OpenZipStorage(ZipFile, "", false, out zipStorage);

                //add files
                if (openFileDialog1.ShowDialog() == DialogResult.OK)
                {
                    foreach (String item in openFileDialog1.FileNames)
                    {
                        zipStorage.AddFile(item, Path.Combine("Test_NewFiles", Path.GetFileName(item)), true);
                        MessageBox.Show(this, "Operation has down successfully.\n\nFile was added to Test.zip file (in bin\\Debug folder), into the \"Test_NewFiles\" folder. You can use \"Select File to View\" button to see it or extract file with other program.", "", MessageBoxButtons.OK, MessageBoxIcon.Information);

                    }
                }


            }
            catch (Exception ex)
            {

                MessageBox.Show(ex.ToString());
            }
            finally
            {
                //ensure storage closed and release lock on file, it not necessary for most case
                if (zipStorage!= null)
                    Marshal.ReleaseComObject(zipStorage);
            }
        }

        private void AddFolderButton_Click(object sender, EventArgs e)
        {
            Shetab.Packer.IZipStorage zipStorage=null;
            try
            {
                //Open zip storage
                
                Pakcer.OpenZipStorage(ZipFile, "", false, out zipStorage);

                //add folder
                if (folderBrowserDialog1.ShowDialog() == DialogResult.OK)
                {
                    zipStorage.AddFolder(folderBrowserDialog1.SelectedPath, Path.Combine("Test_NewFiles", Path.GetFileName(folderBrowserDialog1.SelectedPath)), "*.*", true);
                    MessageBox.Show(this, "Operation has down successfully.\n\nFolder was added to Test.zip file (in bin\\Debug folder), into the \"Test_NewFiles\" folder. You can extract test.zip and view it", "", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    
                }


            }
            catch (Exception ex)
            {

                MessageBox.Show(ex.ToString());
            }
            finally
            {
                //ensure storage closed and release lock on file, it not necessary for most case
                if (zipStorage != null)
                    Marshal.ReleaseComObject(zipStorage);
            }
        }

        /// <summary>
        /// Extract file from zip file using stream
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ExtractFileButton_Click(object sender, EventArgs e)
        {

            try
            {
                String src = "test\\sample.htm"; //is source relative to zipFile


                //Open zip storage
                Shetab.Packer.IZipStorage zipStorage;
                Pakcer.OpenZipStorage(ZipFile, "", true, out zipStorage);

                //check is srcFile exist in zip
                Shetab.Packer.PathExist exist;
                zipStorage.IsPathExists(src, out exist);
                if (exist != Shetab.Packer.PathExist.Stream)
                {
                    MessageBox.Show("File does not exists in zip file!\nFile=" + src);
                    return;
                }


                //Open Stream from Storage
                Shetab.Packer.IBoxStream istream;
                zipStorage.OpenStreamByPath(src, Shetab.Packer.STGM.SHARE_EXCLUSIVE | Shetab.Packer.STGM.READ, out istream);
                Shetab.Packer.IZipStream zipstream = (Shetab.Packer.IZipStream)istream; //don't needed but IZipStream has more functionality

                //get .NET Stream form ComStream
                Shetab.Packer.ComStream stream = new Shetab.Packer.ComStream(istream);

                //select target folder
                saveFileDialog1.FileName = Path.GetFileName(src);
                if (saveFileDialog1.ShowDialog() != DialogResult.OK)
                    return;

                byte[] a = new byte[stream.Length];
                stream.Read(a, 0, a.Length);
                File.WriteAllBytes(saveFileDialog1.FileName, a);
                MessageBox.Show("Operation finished successfully.");

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }

        }

        private void ExtractFolderButton_Click(object sender, EventArgs e)
        {
            try
            {
                String src = "test\\sample"; //source relative to zipFile


                //Open zip storage
                Shetab.Packer.IZipStorage zipStorage;
                Pakcer.OpenZipStorage(ZipFile, "", true, out zipStorage);

                //check is srcFile exist in zip
                Shetab.Packer.PathExist exist;
                zipStorage.IsPathExists(src, out exist);
                if (exist != Shetab.Packer.PathExist.Storage)
                {
                    MessageBox.Show("Folder does not exists in zip file!\nFolder=" + src);
                    return;
                }

                // find target storage
                Shetab.Packer.IBoxStorage targetBoxStorage;
                zipStorage.OpenStorageByPath(src, Shetab.Packer.STGM.SHARE_EXCLUSIVE | Shetab.Packer.STGM.READ, out targetBoxStorage);
                Shetab.Packer.IZipStorage targetZipStorage = (Shetab.Packer.IZipStorage)targetBoxStorage;

                //select target folder
                if (folderBrowserDialog1.ShowDialog() != DialogResult.OK)
                    return;

                string path = Path.Combine(folderBrowserDialog1.SelectedPath, src);
                targetZipStorage.ExtractTo(path, "*.*", true);
                MessageBox.Show("Operation finished successfully.");

            }
            catch (Exception ex)
            {

                MessageBox.Show(ex.ToString());
                
            }

        }

        private void ExtractFileIoButton_Click(object sender, EventArgs e)
        {
            try
            {
                // just mount
                Mount();

                ExtractForm extractForm = new ExtractForm();
                if (extractForm.ShowDialog() == DialogResult.OK)
                {
                    System.IO.File.Copy(extractForm.txtSource.Text, extractForm.txtDestination.Text);
                    MessageBox.Show("Operation finished successfully.");

                }

            }
            catch (Exception ex)
            {

                MessageBox.Show(ex.ToString());
               
            }

        }

        //simple copy directory with standard .net objects
        private void copyDirectory(string Src, string Dst)
        {
            String[] Files;

            if (Dst[Dst.Length - 1] != Path.DirectorySeparatorChar)
                Dst += Path.DirectorySeparatorChar;

            if (!Directory.Exists(Dst)) Directory.CreateDirectory(Dst);
            Files = Directory.GetFileSystemEntries(Src);
            foreach (string Element in Files)
            {
                // Sub directories
                if (Directory.Exists(Element))
                    copyDirectory(Element, Dst + Path.GetFileName(Element));
                // Files in directory
                else
                    File.Copy(Element, Dst + Path.GetFileName(Element), true);
            }
        }

        private void ExtractFolderIoButton_Click(object sender, EventArgs e)
        {

            try
            {
                // just mount
                Mount();

                ExtractFolder extractFolder = new ExtractFolder();
                if (extractFolder.ShowDialog() == DialogResult.OK)
                {
                    string target = Path.Combine(extractFolder.txtDestination.Text, Path.GetFileName(extractFolder.txtSource.Text));
                    copyDirectory(extractFolder.txtSource.Text, target);
                    MessageBox.Show("Operation finished successfully.");


                }
            }
            catch (Exception ex)
            {

                MessageBox.Show(ex.ToString());
                
            }
        }
    }
}
